﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 이벤트를 설정한다.

            this.calculateButton.Click += calculateButton_Click;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 계산하기 버튼 클릭시 처리하기 - calculateButton_Click(sender, e)

        /// <summary>
        /// 계산하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void calculateButton_Click(object sender, EventArgs e)
        {
            Fraction sourceFraction = new Fraction(this.fractionTextBox.Text);

            if(sourceFraction < 0)
            {
                sourceFraction = -sourceFraction;
            }

            List<Fraction> fractionList = GetEgyptianFraction(sourceFraction);

            StringBuilder stringBuilder = new StringBuilder();

            foreach(Fraction fraction in fractionList)
            {
                if(stringBuilder.Length > 0)
                {
                    stringBuilder.Append(" + ");
                }

                stringBuilder.Append(fraction.ToString());
            }

            this.resultTextBox.Text = stringBuilder.ToString();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 이집트 분수 구하기 - GetEgyptianFraction(sourceFraction)

        /// <summary>
        /// 이집트 분수 구하기
        /// </summary>
        /// <param name="sourceFraction">소스 분수</param>
        /// <returns>분수 리스트</returns>
        private List<Fraction> GetEgyptianFraction(Fraction sourceFraction)
        {
            List<Fraction> resultFractionList = new List<Fraction>();

            int wholePart = (int)(sourceFraction.Numerator / sourceFraction.Denominator);

            if(wholePart > 0)
            {
                resultFractionList.Add(wholePart);

                sourceFraction = sourceFraction - wholePart;
            }

            long denominator = 2;

            while(sourceFraction > 0)
            {
                Fraction fraction = new Fraction(1, denominator);

                while(fraction > sourceFraction)
                {
                    denominator++;

                    fraction = new Fraction(1, denominator);
                }

                resultFractionList.Add(fraction);

                sourceFraction -= fraction;

                denominator++;
            }

            return resultFractionList;
        }

        #endregion
    }
}